{
  config,
  lib,
  fn,
  pkgs,
  ...
}:
with lib;
let
  withDocker = config.virtualisation.docker.enable;
  withPodman = config.virtualisation.podman.enable;
  users = user: {
    inherit (user) name;
    value =
      let
        cfg = config.services;
        passPath = config.sops.secrets."users/${user.name}/password".path;
      in
      {
        isNormalUser = true;
        inherit (user) name;
        uid = user.id;
        subUidRanges = optional (user.isAdmin && withPodman) {
          startUid = 100000;
          count = 65536;
        };
        subGidRanges = optional (user.isAdmin && withPodman) {
          startGid = 100000;
          count = 65536;
        };
        home = builtins.toPath "/home/${user.name}";
        createHome = true;
        group = user.name;
        extraGroups =
          [
            "audio"
            "network"
          ]
          ++ (optional user.isAdmin "wheel")
          ++ (optionals (lib.elem "desktop" config.machine.services) [
            "input"
            "video"
          ])
          ++ (optionals cfg.printing.enable [
            "cups"
            "lp"
          ])
          ++ (optional config.programs.virt-manager.enable "libvirtd")
          ++ (optional config.virtualisation.virtualbox.host.enable "vboxusers")
          ++ (optional config.networking.networkmanager.enable "networkmanager")
          ++ (optional (withDocker && !withPodman) "docker")
          ++ (optional withPodman "podman");
        shell = "${pkgs.zsh}/bin/zsh";
        hashedPasswordFile = passPath;
      };
  };

  mkusergroup = user: {
    inherit (user) name;
    value = {
      inherit (user) name;
      gid = user.id;
      members = [ user.name ];
    };
  };
in
{
  sops.secrets = fn.sopsHelper (user: "users/${user.name}/password") config.machine.users {
    neededForUsers = true;
  };
  users = {
    mutableUsers = false;
    users = listToAttrs (map users config.machine.users);
    groups = listToAttrs (map mkusergroup config.machine.users);
  };
}
